///////////////////////////////////////////////////////////////////////////
//
// Debug options for the powertester application.
//
// (C) 2005, Klaus S. Madsen <klaussm@diku.dk>
//

#ifndef POWERTEST_DEBUG_H
#define POWERTEST_DEBUG_H

//
// UART_DEBUG makes use of the StdOutC component, which in turn
// requires that the Atmel processor never enters a deeper sleep than
// idle-mode. Therefore this needs to be disabled in the final
// PowerTester application.
//
//#define UART_DEBUG

//
// UART_ACCEL_DEBUG will print the individual readings from the
// accelerometers. Only useful when UART_DEBUG is also enabled.
//
//#define UART_ACCEL_DEBUG

//
// PACKET_LEAK_DEBUG makes sure that the node does not re-init the
// BTPacketHandler each time it turns on the Bluetooth device. This
// way it is easier to see if we leak packets.
//
//#define PACKET_LEAK_DEBUG

//
// LED_DEBUG allows the use of the btnode onboard leds to indicate the
// "mode" of the node. The leds use around 20mW when turned on, but
// otherwise they use no power. Doesn't necessarily need to be
// disabled in the final PowerTester application, as they are only
// turned on when the node have turned on the BT-radio.
//
//#define LED_DEBUG

//
// DISABLE_FLASH will remove all the calls to the FlashAccess
// component, which will actually write to the flash. This serves two
// purposes:
// 
//  1. Testing. There is no problems if the code writes 100.000 times
//  to the same page, if no writes is actually performed.
//
//  2. Speed. Uploading a version of the code with flash-writing
//  enabled takes around 1 minute. This is a long time to wait in
//  order to test a bugfix
//
//#define DISABLE_FLASH

//
// USE_SPI is used to select how the node communicates with the 3-axis
// accelerometer. If USE_SPI is defined, the SPI-interface is
// used. Otherwise the I2C interface is used.
//
#define USE_SPI

//
// FIRST_SEND_USED_PAGES defines how many pages must be written before
// the node turns the bluetooth radio on for the first time. This a
// sort of a hack to make the node checkin earlier, when we are out in
// the pen. If set to around 10 minutes, we should be able to pack the
// node into the box, and we won't have to wait a whole hour just to
// make sure that all the nodes work.
//
// 85 pages should make for around 10 minutes.
//
#define FIRST_SEND_USED_PAGES 85

//
// When LED_POWER_TEST is defined, the node will turn on all the leds,
// thus consuming more power than what is normaly is used. Otherwise
// it will perform the same as it normaly does. I.e. almost one hour
// between checkins etc.
//
// Also the node is instructed to turn-off when it reboots due to a
// brown-out reset.
//
// This option is incompatible with LED_DEBUG (it uses the same leds).
//
//#define LED_POWER_TEST

//
// When BT_POWER_TEST is enabled, the node will turn on the bluetooth
// when it has accumulated BT_POWER_TEST_PAGES worth of data.
//
// Just as with LED_POWER_TEST the node will be instructed to turn-off
// when it reboots due to a brown-out reset.
//
//#define BT_POWER_TEST

#define BT_POWER_TEST_PAGES 17 // Around 2 minutes

// Test to see if the choices are possible
#if defined LED_POWER_TEST && defined LED_DEBUG
#error Cannot define both LED_DEBUG and LED_POWER_TEST
#endif

#if defined UART_ACCEL_DEBUG && !defined UART_DEBUG
#error Cannot enable UART_ACCEL_DEBUG without UART_DEBUG being enabled
#endif

// Test if the choices are sane
#if (defined LED_POWER_TEST || defined BT_POWER_TEST) && defined UART_DEBUG
#warning Running powertest with uart enabled
#endif

#if (defined LED_POWER_TEST || defined BT_POWER_TEST) && defined DISABLE_FLASH
#warning Running powertest with flash writing disabled.
#endif

#endif

