///////////////////////////////////////////////////////////////////////////
//
// protocol.h 
//
// This file defines the protocol between the node and the server.
//
// (C) Klaus S. Madsen <klaussm@diku.dk>
//

#ifndef _PROTOCOL_H
#define _PROTOCOL_H

//
// The communication between the server and the node is initiated by
// the server. When the server discovers that a new node is present,
// it simply connects to this node.
//
// Once the connection is established, the node sends the n2s_init
// packet.
//
// The server responds with a list of s2n_page_req structures, the
// last one having first_page and last_page set to -1.
//
// Once the node have recieved this list, it sends each of the pages
// asked for by the server in the n2s_page struct. When it has sent
// all these pages, n2s_done is sent.
//
// At this point the server can either send an empty list (containing
// a single s2n_page_req with first_page and last_page set to -1) in
// order to make the node disconnect, or it can send a new list of
// pages for the node to send (or resend).
//

struct n2s_init {
  uint32_t time;
  uint32_t first_page_time;
  uint32_t digi_data_points;
  uint32_t analog_data_points;

  // Difference since last retrieval
  uint16_t put_failures;
  uint16_t get_failures;
  uint16_t bluetooth_errors;
  uint16_t write_page_failures;
  uint8_t reboots;
  uint16_t data_pages_used;

  uint16_t power;
} __attribute__((packed));

struct s2n_page_req {
  int16_t first_page;
  int16_t last_page;
} __attribute__((packed));

#ifndef SPM_PAGESIZE
#define SPM_PAGESIZE 256
#endif

struct n2s_page {
  int16_t page;
  uint8_t data[SPM_PAGESIZE];  
} __attribute__((packed));

struct s2p_done {
  int16_t end_marker; // Must be -1
} __attribute__((packed));

#endif
