#!/usr/bin/perl -w
use POSIX;
use Time::Local;

$filename = shift(@ARGV);

if (!defined $filename || $filename eq "") {
    die "Usage $0 <filename> <start_time> <end_time>";
}

$day = shift(@ARGV);
if (!defined $day) {
    $day = "ALL";
} else {
    $end_time = timelocal(0 , 0, 0, $day + 1, 2, 2005);
    $start_time = timelocal(0 , 0, 0, $day, 2, 2005);

    print "Extracting from " . $start_time . " to " . $end_time . "\n";
}

$basefilename = $filename;
$basefilename =~ s/.accel.csv//g;

open(DUMP, $filename) || die "Unable to open dump file";
open(DIGITAL1, ">$basefilename.$day.digital.input") || die "Unable to open digital file";
open(DIGITAL2, ">$basefilename.$day.digital.bin.input") || die "Unable to open digital file";
open(DIGITAL3, ">$basefilename.$day.digital.time.csv") || die "Unable to open digital file";
open(ANALOG1, ">$basefilename.$day.analog.input") || die "Unable to open digital file";
open(ANALOG2, ">$basefilename.$day.analog.bin.input") || die "Unable to open digital file";
open(ANALOG3, ">$basefilename.$day.analog.time.csv") || die "Unable to open digital file";
open(MISSING, ">$basefilename.$day.missing") || die "Unable to open missing file";

$count = 0;

sub write_data_point
{
    ($ldate, $lsample, $lanalog, $ldigital) = @_;

#    print "Write data point called\n";

    print DIGITAL1 $ldigital . "\n";
    if ($ldigital > 20) {
	print DIGITAL2 "1\n";
    } else {
	print DIGITAL2 "0\n";
    }
    print DIGITAL3 $ldate . ";" . ($lsample - $first_sample_no) . ";" . $ldigital . "\n";

    print ANALOG1 $lanalog . "\n";
    if ($lanalog > 3) {
	print ANALOG2 "1\n";
    } else {
	print ANALOG2 "0\n";
    }
    print ANALOG3 $ldate . ";" . ($lsample - $first_sample_no) . ";" . $lanalog . "\n";
}

while(<DUMP>) {
    if (m|^([^;]+);([^;]+);([^;]+);([^;]+);([^;]+);([^;]+);([^;]+)|) {
	$date = $1;
	$sample = $2;
	$digital_x = $3;
	$digital_y = $4;
	$digital_z = $5;
	$analog_x = $6;
	$analog_y = $7;

	if (defined($end_time) && $date >= $end_time) {
	    print "Found last data point\n";
	    last;
	}

	if (defined($start_time) && $date < $start_time) {
	    next;
	}

	if (!defined $first_sample_no) {
	    print "Found first data point\n";
	    $first_sample_no = $sample;
	}

	if (defined $last_sample) {
	    if ($last_sample + 1 != $sample) {
		for ($i = $last_sample + 1; $i < $sample; $i++) {
		    $analog = 0;
		    $digital = 0;
		    write_data_point("N/A", $i, $analog, $digital);
		}

		print MISSING "" . ($last_sample + 1 - $first_sample_no) 
		    . "," . ($sample - 1 - $first_sample_no) ."\n";
	    }
	    $diff_digi_x = $last_digi_x - $digital_x;
	    $diff_digi_y = $last_digi_y - $digital_y;
	    $diff_digi_z = $last_digi_z - $digital_z;
	    $diff_analog_x = $last_analog_x - $analog_x;
	    $diff_analog_y = $last_analog_y - $analog_y;

	    $digital = sqrt($diff_digi_x ** 2 + $diff_digi_y ** 2 + $diff_digi_z ** 2);
	    $analog = sqrt($diff_analog_x ** 2 + $diff_analog_y ** 2);
	    
	    write_data_point($date, $sample, $analog, $digital);
	    $count++;
	}


	$last_digi_x = $digital_x;
	$last_digi_y = $digital_y;
	$last_digi_z = $digital_z;
	$last_analog_x = $analog_x;
	$last_analog_y = $analog_y;

	$last_sample = $sample;
    } 
}

if (defined($last_sample)) {
    print MISSING $last_sample;
}

print "Found data points: " . $count . "\n";
