/*
 * STM3Digi.h
 *
 * This file contains definitions needed to communicate with the STM
 * LIS3L02DS accelerometer.
 *
 */

#ifndef __STM3DIGI_H
#define __STM3DIGI_H

TOSH_ASSIGN_OUTPUT_ONLY_PIN(STM_PWR, F, 4);
TOSH_ASSIGN_OUTPUT_ONLY_PIN(STM_CS, D, 6);

enum stm_constants_t {
  I2C_AUTO_INC_REGISTER = 0x80,
  SPI_READ_REGISTER = 0x80,
};

/*
  Registers in the LIS3L02DS
*/
enum stm_registers_t {
  OFFSET_X      = 0x16, /* Digital Offset Trimming for X-Axis */
  OFFSET_Y      = 0x17, /* Digital Offset Trimming for Y-Axis */
  OFFSET_Z      = 0x18, /* Digital Offset Trimming for Z-Axis */
  GAIN_X        = 0x19, /* Digital Gain Trimming for X-Axis */
  GAIN_Y        = 0x1A, /* Digital Gain Trimming for Y-Axis */
  GAIN_Z        = 0x1B, /* Digital Gain Trimming for Z-Axis */
  CTRL_REG1     = 0x20, /* See comment for ctrl_reg1_t */
  CTRL_REG2     = 0x21, /* See comment for ctrl_reg2_t */
  WAKE_UP_CFG   = 0x23, /* See comment for wake_up_cfg_t */
  WAKE_UP_SRC   = 0x24, /* See comment for wake_up_src_t */
  WAKE_UP_ACK   = 0x25, /* Reading resets WAKE_UP_SRC */
  STATUS_REG    = 0x27, /* See comment for status_reg_t */
  OUTX_L        = 0x28, /* Low byte of X axis acceleration */
  OUTX_H        = 0x29, /* High byte of X axis acceleration */
  OUTY_L        = 0x2A, /* Low byte of Y axis acceleration */
  OUTY_H        = 0x2B, /* High byte of Y axis acceleration */
  OUTZ_L        = 0x2C, /* Low byte of Z axis acceleration */
  OUTZ_H        = 0x2D, /* High byte of Z axis acceleration */
  THS_L         = 0x2E, /* Low byte of Wake up acceleration threshold */
  THS_H         = 0x2F, /* High byte of Wake up acceleration threshold */
};

/*
 * Bit meanings for CTRL_REG1
 */
enum stm_ctrl_reg1_t {
  CTRL_REG1_PD1 = 0x80, /* Power Down Control (always set to 0) */
  CTRL_REG1_PD0 = 0x40, /* Power Down Control (1 = Device on, 
			0 = Power-down mode) */
  CTRL_REG1_DF1 = 0x20, /* Decimation Factor Control (00 = decimate by 128, */
  CTRL_REG1_DF0 = 0x10, /* 01 = decimate by 64, 10 = decimate by 32, 11 = by 8) */
  CTRL_REG1_ST  = 0x08, /* Self test (0 = normal mode, 1 = self-test */
  CTRL_REG1_Zen = 0x04, /* Z-Axis enable (0 = disabled, 1 = enable */
  CTRL_REG1_Yen = 0x02, /* Y-Axis enable (0 = disabled, 1 = enable */
  CTRL_REG1_Xen = 0x01, /* X-Axis enable (0 = disabled, 1 = enable */
};

enum stm_ctrl_reg2_t {
  CTRL_REG2_FS   = 0x80, /* Full scale selection (0 = +-2g, 1 = +-6g) */
  CTRL_REG2_BOOT = 0x10, /* Reboot memory content */
  CTRL_REG2_IEN  = 0x08, /* Interrupt Enable (0 = data ready on RDY pad, 
			 1 = int req on RDY pad) */
  CTRL_REG2_DRDY = 0x04, /* Enable Data-Ready generation */
  CTRL_REG2_SIM  = 0x02, /* SPI mode selection (0 = 4-wire, 1 = 3-wire) */
  CTRL_REG2_DAS  = 0x01, /* Data Alignement Selection (0 = 12 bit right justified,
			 1 = 16 bit left justified) */
};

enum stm_wake_up_cfg_t {
  WAKE_UP_CFG_LIR = 0x40, /* Latch interrupt request */
  WAKE_UP_CFG_MZH = 0x20, /* Mask Z High Interrupt */
  WAKE_UP_CFG_MZL = 0x10, /* Mask Z Low Interrupt */
  WAKE_UP_CFG_MYH = 0x08, /* Mask Y High Interrupt */
  WAKE_UP_CFG_MYL = 0x04, /* Mask Y Low Interrupt */
  WAKE_UP_CFG_MXH = 0x02, /* Mask X High Interrupt */
  WAKE_UP_CFG_MXL = 0x01, /* Mask X Low Interrupt */
};

enum stm_wake_up_src_t {
  WAKE_UP_SRC_IA = 0x04,  /* Interrupt Active */
  WAKE_UP_SRC_MZH = 0x20, /* Z high */
  WAKE_UP_SRC_MZL = 0x10, /* Z low */
  WAKE_UP_SRC_MYH = 0x08, /* Y high */
  WAKE_UP_SRC_MYL = 0x04, /* Y low */
  WAKE_UP_SRC_MXH = 0x02, /* X high */
  WAKE_UP_SRC_MXL = 0x01, /* X low */
};

#endif
