/* zzz
 * z
 */

enum {
  PAYLOAD_LEN=4,
  ADDR_LEN=5
};

typedef struct {
  uint8_t payload[PAYLOAD_LEN];
  uint8_t addr[ADDR_LEN];
} __attribute__ ((packed)) nRF_pkt_t;

typedef struct {
  unsigned int      rx_en   : 1; // RX or TX operation
  unsigned int      rf_ch   : 7; // Channel frequency
  unsigned int      rf_pwr  : 2; // RF output power
  unsigned int      xo_f    : 3; // Crystal frequency
  unsigned int      rfdr_sb : 1; // RF data rate (1Mbps requires 16 MHz crystal)
  unsigned int      cm      : 1; // Communicaton mode (Direct or ShockBurst)
  unsigned int      rx2_en  : 1; // Enable two channel receive mode
  //high order bits
} __attribute__ ((packed))	gen_config_t;

/*

The data is stored with LSB first  (this is the opposite of network-order on
the SPI line) that is the LSB is located in the lowest memory address

From Product Spcification
rx_en - RX or TX operation
 0 - transmit, 1 - receive

rf_ch - Channel frequency
 Sets the frequency channel the nRF2401 operates on
  ChannelRF = 2400 MHz + rf_ch * 1 MHz
  Channel2RF = ChannelRF + 8 MHz
 In transmit freqencies may be set between 2400 MHz and 2527 MHz
 In receive freqencies may be set between 2400 MHz and 2524 MHz (2408-2524 for ch2)
 Channels above 83 may only be utilized in certain teritories (ex: Japan)

rf_pwr - RF output power
 D9 D8 P (dBm)
 0  0   -20
 0  1   -10
 1  0    -5
 1  1     0

xo_f - Crystal frequency
 D12 D11 D10 Frequency (MHz)
  0   0   0       4
  0   0   1       8
  0   1   0      12
  0   1   1      16
  1   0   0      20

rfdr_sb - RF data rate
 0 - 250 kbps, 1 - 1 Mbps

cm - Communicaton mode
 0 - direct mode, 1 - shockburst

rx2_en Enable two channel receive mode
 0 - One channel receive, 1 - Two channel receive
 N.B.: receive channel 2 is alway 8 channels above channel 1
 */

typedef struct {
  gen_config_t general_config;

  unsigned int      crc_en:1;  // Enable on-chip CRC generation/checking
  unsigned int      crc_l:1;   // 8 or 16 bit CRC
  unsigned int      addr_w:6;  // Number of address bits (both RX channels)
  uint8_t           addr1[5];  // Up to 5 byte address for RX channel 1
  uint8_t           addr2[5];  // Up to 5 byte address for RX channel 2
  uint8_t           data1_w;   // Length of data payload RX channel 1 (bits)
  uint8_t           data2_w;   // Length of data payload RX channel 2 (bits)
  //uint8_t           test[3];   // reserved for testing - theres no need to send them
  // High order bits
} __attribute__ ((packed))	shock_conf_t;

/*
crc_en  - Enable on-chip CRC generation/checking
0: on-chip CRC generation disabled, 1: on-chip CRC generation enabled

crc_l   - 8 or 16 bit CRC
0: 8 bit CRC, 1: 16 bit CRC

addr_w  - Number of address bits (both RX channels)
Number of bits in ShockBurst address

addr1, addr2   - Up to 5 byte address for RX channel 1 or 2

data1_w, data2_w - Length of data payload section RX channel 1 or 2 in bits
 N.B. The total number of bits in a ShockBurst packet may not exceed 256:
      dataX_w = 256 - addr_w - 8 + crc_l*8

test    - reserved for testing
*/

TOSH_ASSIGN_PIN(nRF_CE, B, 0);
TOSH_ASSIGN_PIN(nRF_CS, F, 6);
TOSH_ASSIGN_PIN(nRF_PWR_UP, D, 7);

TOSH_ASSIGN_PIN(nRF_DR1, D, 0);

// This pin is connected differently on htV0
#define TOSH_MAKE_SPI_nSS_OUTPUT() {}
TOSH_ASSIGN_PIN(SPI_SCK,  B, 1);
TOSH_ASSIGN_PIN(MISO,     B, 3);
TOSH_ASSIGN_PIN(MOSI,     B, 2);

#define SIG_DR1 SIG_INTERRUPT0
#define DR1_INT_CLR  EIFR |= _BV(INTF0)
// External Interrupt Mask
#define DR1_INT_ENABLE EIMSK |= _BV(INT0)
#define DR1_INT_DISABLE EIMSK &= ~_BV(INT0)
// Int on rising edge
#define DR1_INT_SETUP EICRA |= _BV(ISC01) | _BV(ISC00)
